@@ -93,6 +93,7 @@ meta_workspace_free (MetaWorkspace *workspace)
   
   g_free (workspace->work_areas);
 
+  g_list_free (workspace->mru_list);
   g_slist_free (workspace->left_struts);
   g_slist_free (workspace->right_struts);
   g_slist_free (workspace->top_struts);
@@ -111,11 +112,33 @@ meta_workspace_add_window (MetaWorkspace *workspace,
 {
   g_return_if_fail (!meta_workspace_contains_window (workspace, window));
   
+  /* If the window is on all workspaces, we want to add it to all mru
+   * lists, otherwise just add it to this workspaces mru list
+   */
+  if (window->on_all_workspaces) 
+    {
+      if (window->workspaces == NULL)
+        {
+          GList* tmp = window->screen->workspaces;
+          while (tmp)
+            {
+              MetaWorkspace* work = (MetaWorkspace*) tmp->data;
+              if (!g_list_find (work->mru_list, window))
+                work->mru_list = g_list_prepend (work->mru_list, window);
+
+              tmp = tmp->next;
+            }
+        }
+    }
+  else
+    {
+      g_assert (g_list_find (workspace->mru_list, window) == NULL);
+      workspace->mru_list = g_list_prepend (workspace->mru_list, window);
+    }
+
   workspace->windows = g_list_prepend (workspace->windows, window);
   window->workspaces = g_list_prepend (window->workspaces, workspace);
 
-  workspace->mru_list = g_list_append (workspace->mru_list, window);
-
   meta_window_set_current_workspace_hint (window);
   
   meta_window_queue_calc_showing (window);
@@ -141,7 +164,30 @@ meta_workspace_remove_window (MetaWorkspace *workspace,
 
   workspace->windows = g_list_remove (workspace->windows, window);
   window->workspaces = g_list_remove (window->workspaces, workspace);
-  workspace->mru_list = g_list_remove (workspace->mru_list, window);
+
+  /* If the window is on all workspaces, we don't want to remove it
+   * from the MRU list unless this causes it to be removed from all 
+   * workspaces
+   */
+  if (window->on_all_workspaces) 
+    {
+      if (window->workspaces == NULL)
+        {
+          GList* tmp = window->screen->workspaces;
+          while (tmp)
+            {
+              MetaWorkspace* work = (MetaWorkspace*) tmp->data;
+              work->mru_list = g_list_remove (work->mru_list, window);
+
+              tmp = tmp->next;
+            }
+        }
+    }
+  else
+    {
+      workspace->mru_list = g_list_remove (workspace->mru_list, window);
+      g_assert (g_list_find (workspace->mru_list, window) == NULL);
+    }
 
   meta_window_set_current_workspace_hint (window);
   
@@ -215,6 +261,7 @@ meta_workspace_activate_with_focus (MetaWorkspace *workspace,
                                     MetaWindow    *focus_this)
 {
   MetaWorkspace *old;
+  MetaWindow *move_window;
   
   meta_verbose ("Activating workspace %d\n",
                 meta_workspace_index (workspace));
@@ -231,15 +278,46 @@ meta_workspace_activate_with_focus (MetaWorkspace *workspace,
   if (old == NULL)
     return;
 
+  move_window = NULL;
+  if (workspace->screen->display->grab_op == META_GRAB_OP_MOVING ||
+      workspace->screen->display->grab_op == META_GRAB_OP_KEYBOARD_MOVING)
+    move_window = workspace->screen->display->grab_window;
+      
+  if (move_window != NULL)
+    {
+      if (move_window->on_all_workspaces)
+        move_window = NULL; /* don't move it after all */
+
+      /* We put the window on the new workspace, flip spaces,
+       * then remove from old workspace, so the window
+       * never gets unmapped and we maintain the button grab
+       * on it.
+       */
+      if (move_window)
+        {
+          if (!meta_workspace_contains_window (workspace,
+                                               move_window))
+            meta_workspace_add_window (workspace, move_window);
+        }
+    }
+
   meta_workspace_queue_calc_showing (old);
   meta_workspace_queue_calc_showing (workspace);
 
+  if (move_window)
+      /* Removes window from other spaces */
+      meta_window_change_workspace (move_window, workspace);
+
   if (focus_this)
     {
       meta_window_focus (focus_this,
                          meta_display_get_current_time (focus_this->display));
       meta_window_raise (focus_this);
     }
+  else if (move_window)
+    {
+      meta_window_raise (move_window);
+    }
   else
     {
       meta_topic (META_DEBUG_FOCUS, "Focusing default window on new workspace\n");
@@ -453,14 +531,17 @@ ensure_work_areas_validated (MetaWorkspace *workspace)
               if ((i == 0) && (w->struts->right.width > 0))
                 {
                   workspace->right_struts = g_slist_prepend (workspace->right_struts,
-							     &w->struts->right);
+                                                             &w->struts->right);
                 }
 
               if (meta_screen_rect_intersects_xinerama (w->screen,
                                                         &w->struts->right,
                                                         i))
                 {
-                  right_strut = MAX (right_strut, w->struts->right.width);
+                  right_strut = MAX (right_strut, w->struts->right.width - 
+                                     workspace->screen->width + 
+                                     workspace->screen->xinerama_infos[i].width +
+                                     workspace->screen->xinerama_infos[i].x_origin);
                   all_right_strut = MAX (all_right_strut, w->struts->right.width);
                 }
 
@@ -490,7 +571,10 @@ ensure_work_areas_validated (MetaWorkspace *workspace)
                                                         &w->struts->bottom,
                                                         i))
                 {
-                  bottom_strut = MAX (bottom_strut, w->struts->bottom.height);
+                  bottom_strut = MAX (bottom_strut, w->struts->bottom.height - 
+                                     workspace->screen->height + 
+                                     workspace->screen->xinerama_infos[i].height +
+                                     workspace->screen->xinerama_infos[i].y_origin);
                   all_bottom_strut = MAX (all_bottom_strut, w->struts->bottom.height);
                 }
             }
